header {
    package com.evelopers.unimod.analysis;

    import com.evelopers.unimod.analysis.executors.Substitution;
	import com.evelopers.unimod.parser.ConstNode;
}

/**
 * Calculates formula using current Substitution. Use case:
 * <pre><code>
 *   FormulaCalculator calculator = new FormulaCalculator();
 *   calculator.calculate(ast, substitution);
 * </code></pre>
 */
class FormulaCalculator extends TreeParser;
options {
    buildAST = false;
    importVocab = Expr;
	defaultErrorHandler = false;
}

/**
 * Calculates formula using current substitution
 *
 * @param _t AST tree that represents the formula to calculate
 * @param substitution current substitution for variables
 */
calculate[Substitution substitution] returns [boolean value]
{
	AST node = calculate_AST_in;
}
    :   OR
    	{
            value = false;
            for (AST child = node.getFirstChild(); child != null; child = child.getNextSibling()) {
            	if (calculate(child, substitution)) {
            		value = true;
            		break;
            	}
            }
    	}
    |   AND
    	{
            value = true;
            for (AST child = node.getFirstChild(); child != null; child = child.getNextSibling()) {
            	if (! calculate(child, substitution)) {
            		value = false;
            		break;
            	}
            }
    	}
    |   NOT
    	{
    		value = ! calculate(node.getFirstChild(), substitution);
    	}
    |   id:IDENT
        {
    		value = substitution.getBooleanVarValue(id.getText());
    	}
    |   p:(EQUAL | NEQUAL | GE | GT | LE | LT) op1:(IDENT | CONST_NUM) op2:(IDENT | CONST_NUM)
        {
        	ConstNode constNode;
        	String variableName;
    		if (op1.getType() == IDENT) {
    			variableName = op1.getText();
    			constNode = (ConstNode) op2;
    		} else {
    			variableName = op2.getText();
    			constNode = (ConstNode) op1;
    		}
    		value = substitution.getPredicateValue(variableName, p.getType(), constNode.getObjectValue());
        }
    |   TRUE 
    	{
    		value = true;
    	}
    | 	FALSE
    	{
    		value = false;
    	}
    ;
